==> aim-transport-0.3.0.tar.gz: aim-transport-0.3.0/src/auth.c <==
#include "aimtrans.h"

at_buddy buddies=NULL;

void at_buddy_find_stale(size_t t)
{
	at_buddy cur;
	at_buddy prev;
	
	if(t == 0 || buddies == NULL)
		return;

	prev = NULL;
	cur = buddies;
	while(cur)
	{
		if(cur->t+30 < t)
		{
			if(prev == NULL)
			{
				prev = cur->next;
				free(cur);
				buddies = cur = prev;
				prev = NULL;
			}
			else
			{
				prev->next=cur->next;
				free(cur);
			}
		}
		else
		{
			cur = cur->next;
		}
	}

	/*for(prev=NULL,next = buddies;next != NULL; next = next->next)
	{
		if(next->t+30 < t)
		{
			if(prev ==NULL)
				prev=next->next;
			else
				prev->next = next->next;
			buddies=prev;
			if(next != NULL)
			{
				log_debug(ZONE, "[AIM] unlinked a buddy that has expired\n");
				free(next);
			}
		} else {
			prev = next;
		}
	}*/

	return;

}

at_buddy at_auth_find_buddy(at_buddy head, jid j)
{
    at_buddy cur;

    for(cur = head; cur != NULL; cur = cur->next)
    {
        if(cur->to)
            if(jid_cmp(cur->to, j)==0)
                return cur;
    }

    return NULL;
}

char *at_auth_get_buddies(session s, jid from)
{
	at_buddy next;
    at_buddy new;
	char *blist=NULL;
	spool sp;
	int found;
    pool p;

    p = pool_new();
	sp = spool_new(p);

	log_debug(ZONE, "[AIM] Getting buddies for %s\n", jid_full(from));
	if(from == NULL || buddies == NULL)
	{
		log_debug(ZONE, "[AIM] Something is null (probably buddies), exiting.");
		return NULL;
	}
	at_buddy_find_stale(time(NULL));
	for(next = buddies;next != NULL; next = next->next)
	{
		/* Check for borked data */
		if(next == NULL || next->from == NULL)
			break;

		if(j_strcmp(jid_full(next->from),jid_full(from))==0)
		{
			log_debug(ZONE, "[AIM] Adding a buddy %s\n", jid_full(next->to));
			
			spooler(sp, next->to->user, "&", sp);
            new = pmalloc(s->p, sizeof(_at_buddy));
            memcpy(new, next, sizeof(_at_buddy));
            new->next = s->buddies;
            s->buddies = new;

			found = 1;
			next->t = time(NULL)+31;
		}
	}

	if(found)
	{
		blist = spool_print(sp);
		log_debug(ZONE, "[AIM]  Generated a buddy list:  %s\n", blist);
	}
	else
	{
		blist = NULL;
		log_debug(ZONE, "[AIM] User has no buddies\n");
	}
	
    pool_free(p);
	return blist;
}

void at_auth_subscribe_buddy(jpacket jp)
{
    xmlnode dup,dup2;
	jid from,to;
        session s=at_session_find_by_jid(jp->from);

        /* check for a valid session */
        if(s==NULL)
        {
            xmlnode err,error;
            /* if no session, and subscribing to a buddy, not aim-t */
            err=xmlnode_new_tag("message");
            xmlnode_put_attrib(err,"type","error");
            xmlnode_put_attrib(err,"from",at_name);
            xmlnode_put_attrib(err,"to",jid_full(jp->from));
            error=xmlnode_insert_tag(err,"error");
            xmlnode_insert_cdata(error,"Cannot Subscribe to a AIM Buddy without a registration",-1);
            xmlnode_put_attrib(error,"code","407");
            ehandler_send(ehandler_at,err,jp->from->server);
            return;
        }
	/* Create the duplicate for the subscribe notice
	 * Construct a subscribed notification
	 * 
	 * I had to do these in this odd order due to memory concerns
	 * Dunno why I didn't just make two dups.
	 *
	 */
	dup = xmlnode_dup(jp->x);
	dup2 = xmlnode_dup(jp->x);

	xmlnode_put_attrib(dup, "to", jid_full(jp->from));
	xmlnode_put_attrib(dup, "from", jid_full(jp->to));
	xmlnode_put_attrib(dup, "type", "subscribed");
	log_debug(ZONE, "[AIM] Sending subscribed notice\n");
	ehandler_send(ehandler_at, dup, jp->from->server);

	/* Construct the subscribe back */
	xmlnode_put_attrib(dup2, "type", "subscribe");
    xmlnode_put_attrib(dup2, "to", jid_full(jp->from));
    xmlnode_put_attrib(dup2, "from", jid_full(jp->to));
	log_debug(ZONE, "[AIM] Asking for a subscribe\n");
	ehandler_send(ehandler_at, dup2, jp->from->server);

	return;
}

void at_auth_add_buddy(jpacket jp)
{
	session s;
	at_buddy buddy;
	int newbud=0;
	
    log_debug(ZONE, "[AIM] Going to add a buddy\n");
	s = at_session_find_by_jid(jp->from);
	if(s==NULL) {
		newbud = 1;
	}
	else {
		if(s->loggedin == 0)
			newbud=1;
		else
			newbud=0;
	}
		
	if(newbud)
	{
        if(at_auth_find_buddy(buddies, jp->to)!=NULL)
            return;
		log_debug(ZONE, "[AIM] New buddy in queue: %s\n",jid_full(jp->to));
		at_buddy_find_stale(time(NULL));
		buddy = malloc(sizeof(_at_buddy));
		buddy->to=jp->to;
		buddy->from=jp->from;
		buddy->t=time(NULL);
		buddy->next=buddies;
		buddies=buddy;
		return;
	}
	else
	{
        if(at_auth_find_buddy(s->buddies, jp->to)!=NULL)
            return;
		log_debug(ZONE, "[AIM] Adding buddy %s to %s\n", jp->to->user, jid_full(jp->from));
        buddy = pmalloc(s->p, sizeof(_at_buddy));
        buddy->to=jid_new(s->p, jid_full(jp->to));
        buddy->from = jid_new(s->p, jid_full(jp->from));
        buddy->t=0;
        buddy->next = s->buddies;
        s->buddies=buddy;
        at_auth_subscribe_buddy(jp);
		aim_add_buddy(&s->ass, aim_getconn_type(&s->ass, AIM_CONN_TYPE_BOS),
                       buddy->to->user);
	}
}
		
			
void at_sleeper_main(void *arg)
{
	at_login login = (at_login)arg;
	jid j = (jid)(login->from);
	jid from = (jid)(login->to);
    jpacket jp = (jpacket)(login->jp);

	log_debug(ZONE, "[AIM] Sleeping on %s\n", jid_full(j));
	pth_sleep(2);

	at_session_create(login->user, login->pass, j, from);

	pth_yield(NULL);

    xmlnode_free(jp->x);
	free(login);
	
	return;
}

int at_auth_user(xmlnode xp)
{
	char *name;
	char *pass;
    xmlnode res;
	session s;
	pth_attr_t attr;
	at_login login;
    jpacket jp;

	/* Just a backup */
    jp = jpacket_new(xp);
	
	if(jp->to->resource == NULL)
	{
		at_auth_add_buddy(jp);
		return;
	}

	res = (xmlnode)jid_xres(jp->to);

	name = xmlnode_get_attrib(res, "name");
	pass = xmlnode_get_attrib(res, "pass");

	log_debug(ZONE, "[AIM] Auth recevied for %s\n", xmlnode2str(res));

    if(jp == NULL || name == NULL || pass == NULL)
        return 0;

	s = at_session_find_by_jid(jp->from);
	if(s==NULL)
	{
		attr = pth_attr_new();
		pth_attr_set(attr, PTH_ATTR_JOINABLE, FALSE);
		login = malloc(sizeof(_at_login));
		login->user = name;
		login->pass = pass;
		login->from = jp->from;
		login->to = jp->to;
        login->jp = jp;
		pth_spawn(attr, (void *)at_sleeper_main, (void *)login);
		pth_attr_destroy(attr);
		pth_yield(NULL);
		/* Now handled via sleeper thread
		 * 
		 * s = at_session_create(name, pass, jp->from);
		 */
	}
	else
	{
		log_debug(ZONE, "[AIM] Dumbass is already online\n");
	}
    return 0;
}

void at_subscribe_auth(jid to, jid from, char *user, char *pass)
{
	/* Here we actually subscribe to the user */
	xmlnode x;
	pool p;
	spool s;
	/* convert from to be aim.foo.net/registered?name=foo&pass=bar */
        p=pool_new();
        s=spool_new(p);
        spooler(s,at_name,"/registered?name=",user,"&pass=",pass,s);
	log_debug(ZONE, "[AIM] Subscribing to %s presence\n", jid_full(to));
	
	x = xmlnode_new_tag("presence");
	xmlnode_put_attrib(x, "to", jid_full(to));
	xmlnode_put_attrib(x, "from", spool_print(s));
	xmlnode_put_attrib(x, "type", "subscribe");

	ehandler_send(ehandler_at, x, to->server);
        pool_free(p);
	return;
}

==> aim-transport-0.8.0.tar.gz: aim-transport-0.8.0/src/auth.c <==
#include "aimtrans.h"

void at_sleeper_main(void *arg)
{
	at_login login = (at_login)arg;
	jid j = (jid)(login->from);
	jid from = (jid)(login->to);
    jpacket jp = (jpacket)(login->jp);

	log_debug(ZONE, "[AIM] Sleeping on %s\n", jid_full(j));
	pth_sleep(2);

	at_session_create(login->user, login->pass, j, from);

	pth_yield(NULL);

    xmlnode_free(jp->x);
	free(login);
	
	return;
}

int at_auth_user(xmlnode xp)
{
	char *name;
	char *pass;
    xmlnode res;
	session s;
	pth_attr_t attr;
	at_login login;
    jpacket jp;

	/* Just a backup */
    jp = jpacket_new(xp);
	
	if(jp->to->resource == NULL)
	{
		return at_buddy_add(jp);
	}

	res = (xmlnode)jid_xres(jp->to);

	name = xmlnode_get_attrib(res, "name");
	pass = xmlnode_get_attrib(res, "pass");

	log_debug(ZONE, "[AIM] Auth recevied for %s\n", xmlnode2str(res));

    if(jp == NULL || name == NULL || pass == NULL)
        return 0;

	s = at_session_find_by_jid(jp->from);
	if(s==NULL)
	{
		attr = pth_attr_new();
		pth_attr_set(attr, PTH_ATTR_JOINABLE, FALSE);
		login = malloc(sizeof(_at_login));
		login->user = name;
		login->pass = pass;
		login->from = jp->from;
		login->to = jp->to;
        login->jp = jp;
		pth_spawn(attr, (void *)at_sleeper_main, (void *)login);
		pth_attr_destroy(attr);
		pth_yield(NULL);
		/* Now handled via sleeper thread
		 * 
		 * s = at_session_create(name, pass, jp->from);
		 */
	}
	else
	{
		log_debug(ZONE, "[AIM] Dumbass is already online\n");
	}
    return 0;
}

void at_auth_subscribe(jid to, jid from, char *user, char *pass)
{
    /* Here we actually subscribe to the user */
    xmlnode x;
    spool s;
    
    x = xmlnode_new_tag("presence");
    
    /* convert from to be aim.foo.net/registered?name=foo&pass=bar */
    s=spool_new(xmlnode_pool(x));
    spooler(s,at_name,"/registered?name=",user,"&pass=",pass,s);
    log_debug(ZONE, "[AIM] Subscribing to %s presence\n", jid_full(to));
	
    xmlnode_put_attrib(x, "to", jid_full(to));
    xmlnode_put_attrib(x, "from", spool_print(s));
    xmlnode_put_attrib(x, "type", "subscribe");

    ehandler_send(ehandler_at, x, to->server);

    return;
}
